from pathlib import Path


def path_from_dir(relative: Path) -> str:
    parts = []
    for part in relative.parts:
        if part.startswith('(') and part.endswith(')'):
            continue
        elif part.startswith('[') and part.endswith(']'):
            parts.append('{' + part[1:-1] + '}')
        else:
            parts.append(part)
    if len(parts) == 0:
        return ''
    return '/' + '/'.join(parts)



from pathlib import Path
from fastapi import APIRouter
from .builder import build_router_tree


class FileRouter:
    def __init__(self, pages_dir: str = "pages"):
        self.pages_dir = Path(pages_dir).resolve()

    def build(self) -> APIRouter:
        return build_router_tree(self.pages_dir)



import importlib.util
from pathlib import Path
from fastapi import APIRouter
from .exceptions import InvalidRouterError


def import_router_from_file(route_file: Path) -> APIRouter:
    spec = importlib.util.spec_from_file_location("route_module", route_file)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    router = getattr(module, "router", None)
    if not isinstance(router, APIRouter):
        raise InvalidRouterError(route_file)
    return router



from pathlib import Path
from fastapi import APIRouter
from .loader import import_router_from_file
from .path_utils import path_from_dir


def build_router_tree(route_dir: Path) -> APIRouter:
    route_path = route_dir / "route.py"
    router = import_router_from_file(
        route_path) if route_path.exists() else None
    if router is None:
        router = APIRouter()

    for sub_dir in route_dir.iterdir():
        if sub_dir.is_dir():
            sub_router = build_router_tree(sub_dir)
            prefix = path_from_dir(sub_dir.relative_to(route_dir))
            router.include_router(sub_router, prefix=prefix)

    return router
